@@ -7,7 +7,10 @@ from nautobot_device_lifecycle_mgmt.api.views import (
     ProviderLCMView,
     ContactLCMView,
     SoftwareLCMViewSet,
+    SoftwareImageLCMViewSet,
     ValidatedSoftwareLCMViewSet,
+    CVELCMViewSet,
+    VulnerabilityLCMViewSet,
 )
 
 router = routers.DefaultRouter()
@@ -17,7 +20,10 @@ router.register(r"contract", ContractLCMView)
 router.register(r"provider", ProviderLCMView)
 router.register(r"contact", ContactLCMView)
 router.register(r"software", SoftwareLCMViewSet)
+router.register(r"software-image", SoftwareImageLCMViewSet)
 router.register(r"validated-software", ValidatedSoftwareLCMViewSet)
+router.register(r"cve", CVELCMViewSet)
+router.register(r"vulnerability", VulnerabilityLCMViewSet)
 
 app_name = "nautobot_device_lifecycle_mgmt"
 
