@@ -1,14 +1,20 @@
 """Forms implementation for the Lifecycle Management plugin."""
 import logging
 from django import forms
+from django.db.models import Q
 
 from nautobot.dcim.models import Device, DeviceRole, DeviceType, InventoryItem, Platform, Region, Site
 from nautobot.extras.forms import (
     CustomFieldModelCSVForm,
     CustomFieldModelForm,
+    CustomFieldFilterForm,
+    CustomFieldBulkEditForm,
     RelationshipModelForm,
+    StatusBulkEditFormMixin,
+    StatusModelCSVFormMixin,
+    StatusFilterFormMixin,
 )
-from nautobot.extras.models import Tag
+from nautobot.extras.models import Tag, Status
 from nautobot.utilities.forms import (
     BootstrapMixin,
     BulkEditForm,
@@ -19,8 +25,15 @@ from nautobot.utilities.forms import (
     BOOLEAN_WITH_BLANK_CHOICES,
     add_blank_choice,
     CSVModelChoiceField,
+    TagFilterField,
+)
+from nautobot_device_lifecycle_mgmt.choices import (
+    ContractTypeChoices,
+    CurrencyChoices,
+    PoCTypeChoices,
+    CountryCodes,
+    CVESeverityChoices,
 )
-from nautobot_device_lifecycle_mgmt.choices import ContractTypeChoices, CurrencyChoices, PoCTypeChoices, CountryCodes
 from nautobot_device_lifecycle_mgmt.models import (
     HardwareLCM,
     InventoryItemSoftwareValidationResult,
@@ -30,11 +43,26 @@ from nautobot_device_lifecycle_mgmt.models import (
     ContractLCM,
     ProviderLCM,
     ContactLCM,
+    CVELCM,
+    VulnerabilityLCM,
+    SoftwareImageLCM,
 )
 
 logger = logging.getLogger("nautobot_device_lifecycle_mgmt")
 
 
+class CSVMultipleModelChoiceField(forms.ModelMultipleChoiceField):
+    """Reference a list of PKs."""
+
+    def prepare_value(self, value):
+        """Parse a comma-separated string of PKs into a list of PKs."""
+        pk_list = []
+        if isinstance(value, str):
+            pk_list = [val.strip() for val in value.split(",") if val]
+
+        return super().prepare_value(pk_list)
+
+
 class HardwareLCMForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm):
     """Hardware Device Lifecycle creation/edit form."""
 
@@ -207,9 +235,6 @@ class SoftwareLCMFilterForm(BootstrapMixin, forms.ModelForm):
             "end_of_support_before",
             "end_of_support_after",
             "documentation_url",
-            "download_url",
-            "image_file_name",
-            "image_file_checksum",
             "long_term_support",
             "pre_release",
         ]
@@ -237,6 +262,161 @@ class SoftwareLCMCSVForm(CustomFieldModelCSVForm):
         fields = SoftwareLCM.csv_headers
 
 
+class SoftwareImageLCMForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm):
+    """SoftwareImageLCM creation/edit form."""
+
+    software = DynamicModelChoiceField(queryset=SoftwareLCM.objects.all(), required=True)
+    device_types = DynamicModelMultipleChoiceField(queryset=DeviceType.objects.all(), required=False)
+    inventory_items = DynamicModelMultipleChoiceField(queryset=InventoryItem.objects.all(), required=False)
+    object_tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
+
+    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
+
+    class Meta:
+        """Meta attributes."""
+
+        model = SoftwareImageLCM
+        fields = (
+            *SoftwareImageLCM.csv_headers,
+            "tags",
+        )
+
+    def clean(self):  # pylint: disable=too-many-locals,too-many-branches
+        """Custom validation of the SoftwareImageLCMForm."""
+        super().clean()
+        device_types = self.cleaned_data.get("device_types")
+        inventory_items = self.cleaned_data.get("inventory_items")
+        object_tags = self.cleaned_data.get("object_tags")
+        default_image = self.cleaned_data.get("default_image")
+        software = self.cleaned_data.get("software")
+
+        if software:
+            software_images = SoftwareImageLCM.objects.filter(software=software)
+            software_default_image = software_images.filter(default_image=True)
+            if self.instance is not None and self.instance.pk is not None:
+                software_images = software_images.filter(~Q(pk=self.instance.pk))
+                software_default_image = software_default_image.filter(~Q(pk=self.instance.pk))
+
+        if software and default_image and software_default_image.exists():
+            msg = "Only one default Software Image is allowed for each Software."
+            self.add_error("default_image", msg)
+
+        assigned_objects_count = sum(obj.count() for obj in (device_types, inventory_items, object_tags))
+        if default_image and assigned_objects_count > 0:
+            msg = "Default image cannot be assigned to any objects."
+            self.add_error("default_image", msg)
+            if device_types.count() > 0:
+                self.add_error("device_types", msg)
+            if inventory_items.count() > 0:
+                self.add_error("inventory_items", msg)
+            if object_tags.count() > 0:
+                self.add_error("object_tags", msg)
+
+        if software and assigned_objects_count > 0:
+            software_manufacturer = software.device_platform.manufacturer
+            for device_type in device_types:
+                if device_type.manufacturer != software_manufacturer:
+                    msg = f"Manufacturer for {device_type.model} doesn't match the Software Platform Manufacturer."
+                    self.add_error("device_types", msg)
+
+                software_img_for_dt = software_images.filter(device_types__in=[device_type])
+                if software_img_for_dt.exists():
+                    msg = f"Device Type {device_type.model} already assigned to another Software Image."
+                    self.add_error("device_types", msg)
+                    self.add_error(None, msg)
+
+            for object_tag in object_tags:
+                software_img_for_tag = software_images.filter(object_tags__in=[object_tag])
+                if software_img_for_tag.exists():
+                    msg = f"Object Tag {object_tag.name} already assigned to another Software Image."
+                    self.add_error("object_tags", msg)
+                    self.add_error(None, msg)
+
+            for inventory_item in inventory_items:
+                software_img_for_invitem = software_images.filter(inventory_items__in=[inventory_item])
+                if software_img_for_invitem.exists():
+                    msg = f"Inventory Item {inventory_item.name} already assigned to another Software Image."
+                    self.add_error("inventory_items", msg)
+                    self.add_error(None, msg)
+
+
+class SoftwareImageLCMFilterForm(BootstrapMixin, forms.ModelForm):
+    """Filter form to filter searches for SoftwareImageLCM."""
+
+    q = forms.CharField(
+        required=False,
+        label="Search",
+        help_text="Search for image name or software version.",
+    )
+    software = DynamicModelMultipleChoiceField(required=False, queryset=SoftwareLCM.objects.all())
+    image_file_name = forms.CharField(
+        required=False,
+        label="Image File name",
+    )
+    device_types = DynamicModelMultipleChoiceField(
+        queryset=DeviceType.objects.all(),
+        to_field_name="model",
+        required=False,
+    )
+    inventory_items = DynamicModelMultipleChoiceField(
+        queryset=InventoryItem.objects.all(),
+        to_field_name="id",
+        required=False,
+    )
+    object_tags = DynamicModelMultipleChoiceField(
+        queryset=Tag.objects.all(),
+        to_field_name="slug",
+        required=False,
+    )
+    default_image = forms.BooleanField(required=False, widget=StaticSelect2(choices=BOOLEAN_WITH_BLANK_CHOICES))
+
+    class Meta:
+        """Meta attributes."""
+
+        model = SoftwareImageLCM
+        fields = [
+            "q",
+            "software",
+            "image_file_name",
+            "image_file_checksum",
+            "download_url",
+            "device_types",
+            "inventory_items",
+            "object_tags",
+            "default_image",
+        ]
+
+        widgets = {
+            "default_image": StaticSelect2(choices=BOOLEAN_WITH_BLANK_CHOICES),
+        }
+
+
+class SoftwareImageLCMCSVForm(CustomFieldModelCSVForm):
+    """Form for bulk creating SoftwareImageLCM objects."""
+
+    device_types = CSVMultipleModelChoiceField(
+        queryset=DeviceType.objects.all(),
+        required=False,
+        to_field_name="model",
+        help_text="Comma-separated list of DeviceType Models",
+    )
+    inventory_items = CSVMultipleModelChoiceField(
+        queryset=InventoryItem.objects.all(),
+        required=False,
+        to_field_name="id",
+        help_text="Comma-separated list of InventoryItem IDs",
+    )
+    object_tags = CSVMultipleModelChoiceField(
+        queryset=Tag.objects.all(), required=False, to_field_name="slug", help_text="Comma-separated list of Tag Slugs"
+    )
+
+    class Meta:
+        """Meta attributes for the SoftwareImageLCMCSVForm class."""
+
+        model = SoftwareImageLCM
+        fields = SoftwareImageLCM.csv_headers
+
+
 class ValidatedSoftwareLCMForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm):
     """ValidatedSoftwareLCM creation/edit form."""
 
@@ -466,18 +646,6 @@ class InventoryItemSoftwareValidationResultFilterForm(BootstrapMixin, CustomFiel
         ]
 
 
-class CSVMultipleModelChoiceField(forms.ModelMultipleChoiceField):
-    """Reference a list of PKs."""
-
-    def prepare_value(self, value):
-        """Parse a comma-separated string of PKs into a list of PKs."""
-        pk_list = []
-        if isinstance(value, str):
-            pk_list = [val.strip() for val in value.split(",") if val]
-
-        return super().prepare_value(pk_list)
-
-
 class ValidatedSoftwareLCMCSVForm(CustomFieldModelCSVForm):
     """Form for bulk creating ValidatedSoftwareLCM objects."""
 
@@ -809,3 +977,188 @@ class ContactLCMCSVForm(CustomFieldModelCSVForm):
 
         model = ContactLCM
         fields = ContactLCM.csv_headers
+
+
+class CVELCMForm(
+    StatusBulkEditFormMixin, BootstrapMixin, CustomFieldModelForm, RelationshipModelForm
+):  # pylint: disable=too-many-ancestors
+    """CVE Lifecycle Management creation/edit form."""
+
+    published_date = forms.DateField(widget=DatePicker())
+    severity = forms.ChoiceField(choices=CVESeverityChoices.CHOICES, label="Severity", required=False)
+    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
+
+    model = CVELCM
+
+    class Meta:
+        """Meta attributes for the CVELCMForm class."""
+
+        model = CVELCM
+
+        fields = [
+            *CVELCM.csv_headers,
+            "tags",
+        ]
+
+        widgets = {
+            "published_date": DatePicker(),
+        }
+
+
+class CVELCMBulkEditForm(StatusBulkEditFormMixin, BootstrapMixin, CustomFieldBulkEditForm):
+    """CVE Lifecycle Management bulk edit form."""
+
+    model = CVELCM
+    pk = forms.ModelMultipleChoiceField(queryset=CVELCM.objects.all(), widget=forms.MultipleHiddenInput)
+    description = forms.CharField(required=False)
+    comments = forms.CharField(required=False)
+    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
+
+    class Meta:
+        """Meta attributes for the CVELCMBulkEditForm class."""
+
+        nullable_fields = [
+            "description",
+            "comments",
+            "status",
+            "tags",
+        ]
+
+
+class CVELCMFilterForm(BootstrapMixin, StatusFilterFormMixin, CustomFieldFilterForm):
+    """Filter form to filter searches for CVELCM."""
+
+    model = CVELCM
+    q = forms.CharField(
+        required=False,
+        label="Search",
+        help_text="Search for name or link.",
+    )
+    severity = forms.ChoiceField(
+        widget=StaticSelect2,
+        required=False,
+        choices=add_blank_choice(CVESeverityChoices.CHOICES),
+    )
+
+    published_date_before = forms.DateField(label="Published Date Before", required=False, widget=DatePicker())
+    published_date_after = forms.DateField(label="Published Date After", required=False, widget=DatePicker())
+
+    cvss__gte = forms.FloatField(label="CVSS Score Above", required=False)
+    cvss__lte = forms.FloatField(label="CVSS Score Below", required=False)
+
+    cvss_v2__gte = forms.FloatField(label="CVSSv2 Score Above", required=False)
+    cvss_v2__lte = forms.FloatField(label="CVSSv2 Score Below", required=False)
+
+    cvss_v3__gte = forms.FloatField(label="CVSSv3 Score Above", required=False)
+    cvss_v3__lte = forms.FloatField(label="CVSSv3 Score Below", required=False)
+
+    status = DynamicModelMultipleChoiceField(queryset=Status.objects.all(), required=False, to_field_name="slug")
+    exclude_status = DynamicModelMultipleChoiceField(
+        label="Exclude Status",
+        required=False,
+        queryset=Status.objects.all(),
+        query_params={"content_types": model._meta.label_lower},
+        to_field_name="slug",
+    )
+    tag = TagFilterField(model)
+
+    class Meta:
+        """Meta attributes."""
+
+        model = CVELCM
+        fields = [
+            "q",
+            "published_date_before",
+            "published_date_after",
+            "severity",
+            "status",
+        ]
+
+
+class CVELCMCSVForm(CustomFieldModelCSVForm, StatusModelCSVFormMixin):
+    """Form for creating bulk CVEs."""
+
+    severity = forms.ChoiceField(choices=CVESeverityChoices.CHOICES, label="CVE Severity")
+
+    class Meta:
+        """Meta attributes for the CVELCMCSVForm class."""
+
+        model = CVELCM
+        fields = CVELCM.csv_headers
+
+
+class VulnerabilityLCMForm(
+    StatusBulkEditFormMixin, BootstrapMixin, CustomFieldModelForm, RelationshipModelForm
+):  # pylint: disable=too-many-ancestors
+    """Vulnerability Lifecycle Management creation/edit form."""
+
+    model = VulnerabilityLCM
+    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
+
+    class Meta:
+        """Meta attributes for the VulnerabilityLCMForm class."""
+
+        model = VulnerabilityLCM
+
+        fields = [
+            "status",
+            "tags",
+        ]
+
+
+class VulnerabilityLCMBulkEditForm(StatusBulkEditFormMixin, BootstrapMixin, CustomFieldBulkEditForm):
+    """Vulnerability Lifecycle Management bulk edit form."""
+
+    model = VulnerabilityLCM
+    pk = forms.ModelMultipleChoiceField(queryset=VulnerabilityLCM.objects.all(), widget=forms.MultipleHiddenInput)
+    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
+
+    class Meta:
+        """Meta attributes for the VulnerabilityLCMBulkEditForm class."""
+
+        nullable_fields = [
+            "status",
+            "tags",
+        ]
+
+
+class VulnerabilityLCMFilterForm(BootstrapMixin, StatusFilterFormMixin, CustomFieldFilterForm):
+    """Filter form to filter searches for VulnerabilityLCM."""
+
+    model = VulnerabilityLCM
+    q = forms.CharField(
+        required=False,
+        label="Search",
+        help_text="Search for name or link.",
+    )
+    cve = DynamicModelMultipleChoiceField(required=False, queryset=CVELCM.objects.all(), label="CVE")
+    cve__published_date__lte = forms.DateField(label="CVE Published Date Before", required=False, widget=DatePicker())
+    cve__published_date__gte = forms.DateField(label="CVE Published Date After", required=False, widget=DatePicker())
+    cve__severity = forms.ChoiceField(
+        label="CVE Severity",
+        widget=StaticSelect2,
+        required=False,
+        choices=add_blank_choice(CVESeverityChoices.CHOICES),
+    )
+    software = DynamicModelMultipleChoiceField(required=False, queryset=SoftwareLCM.objects.all())
+    device = DynamicModelMultipleChoiceField(required=False, queryset=Device.objects.all())
+    inventory_item = DynamicModelMultipleChoiceField(required=False, queryset=InventoryItem.objects.all())
+    status = DynamicModelMultipleChoiceField(queryset=Status.objects.all(), required=False, to_field_name="slug")
+    exclude_status = DynamicModelMultipleChoiceField(
+        label="Exclude Status",
+        required=False,
+        queryset=Status.objects.all(),
+        query_params={"content_types": model._meta.label_lower},
+        to_field_name="slug",
+    )
+    tag = TagFilterField(model)
+
+    class Meta:
+        """Meta attributes."""
+
+        model = VulnerabilityLCM
+        fields = [
+            "q",
+            *VulnerabilityLCM.csv_headers,
+            "tags",
+        ]
