@@ -7,6 +7,7 @@ from django.urls import reverse
 from django.core.exceptions import ValidationError
 from django.conf import settings
 from nautobot.extras.utils import extras_features
+from nautobot.extras.models.statuses import StatusField
 from nautobot.core.models.generics import PrimaryModel, OrganizationalModel
 from nautobot.dcim.models import Device, InventoryItem
 from nautobot.utilities.querysets import RestrictedQuerySet
@@ -15,6 +16,10 @@ from nautobot_device_lifecycle_mgmt import choices
 from nautobot_device_lifecycle_mgmt.software_filters import (
     DeviceValidatedSoftwareFilter,
     InventoryItemValidatedSoftwareFilter,
+    DeviceSoftwareFilter,
+    InventoryItemSoftwareFilter,
+    DeviceSoftwareImageFilter,
+    InventoryItemSoftwareImageFilter,
 )
 
 
@@ -148,6 +153,23 @@ class HardwareLCM(PrimaryModel):
         )
 
 
+class SoftwareLCMQuerySet(RestrictedQuerySet):
+    """Queryset for `SoftwareLCM` objects."""
+
+    def get_for_object(self, obj):
+        """Return all `SoftwareLCM` assigned to the given object."""
+        if not isinstance(obj, models.Model):
+            raise TypeError(f"{obj} is not an instance of Django Model class")
+        if isinstance(obj, Device):
+            qs = DeviceSoftwareFilter(qs=self, item_obj=obj).filter_qs()
+        elif isinstance(obj, InventoryItem):
+            qs = InventoryItemSoftwareFilter(qs=self, item_obj=obj).filter_qs()
+        else:
+            qs = self
+
+        return qs
+
+
 @extras_features(
     "custom_fields",
     "custom_links",
@@ -167,9 +189,6 @@ class SoftwareLCM(PrimaryModel):
     release_date = models.DateField(null=True, blank=True, verbose_name="Release Date")
     end_of_support = models.DateField(null=True, blank=True, verbose_name="End of Software Support")
     documentation_url = models.URLField(blank=True, verbose_name="Documentation URL")
-    download_url = models.URLField(blank=True, verbose_name="Download URL")
-    image_file_name = models.CharField(blank=True, max_length=100, verbose_name="Image File Name")
-    image_file_checksum = models.CharField(blank=True, max_length=256, verbose_name="Image File Checksum")
     long_term_support = models.BooleanField(verbose_name="Long Term Support", default=False)
     pre_release = models.BooleanField(verbose_name="Pre-Release", default=False)
 
@@ -180,9 +199,6 @@ class SoftwareLCM(PrimaryModel):
         "release_date",
         "end_of_support",
         "documentation_url",
-        "download_url",
-        "image_file_name",
-        "image_file_checksum",
         "long_term_support",
         "pre_release",
     ]
@@ -214,13 +230,96 @@ class SoftwareLCM(PrimaryModel):
             self.release_date,
             self.end_of_support,
             self.documentation_url,
-            self.download_url,
-            self.image_file_name,
-            self.image_file_checksum,
             self.long_term_support,
             self.pre_release,
         )
 
+    objects = SoftwareLCMQuerySet.as_manager()
+
+
+class SoftwareImageLCMQuerySet(RestrictedQuerySet):
+    """Queryset for `SoftwareImageLCM` objects."""
+
+    def get_for_object(self, obj):
+        """Return all `SoftwareImageLCM` assigned to the given object."""
+        if not isinstance(obj, models.Model):
+            raise TypeError(f"{obj} is not an instance of Django Model class")
+        if isinstance(obj, Device):
+            qs = DeviceSoftwareImageFilter(qs=self, item_obj=obj).filter_qs()
+        elif isinstance(obj, InventoryItem):
+            qs = InventoryItemSoftwareImageFilter(qs=self, item_obj=obj).filter_qs()
+        else:
+            qs = self
+
+        return qs
+
+
+@extras_features(
+    "custom_fields",
+    "custom_links",
+    "custom_validators",
+    "export_templates",
+    "graphql",
+    "relationships",
+    "statuses",
+    "webhooks",
+)
+class SoftwareImageLCM(PrimaryModel):
+    """SoftwareImageLCM model."""
+
+    image_file_name = models.CharField(blank=False, max_length=100, verbose_name="Image File Name")
+    software = models.ForeignKey(
+        to="SoftwareLCM", on_delete=models.CASCADE, related_name="software_images", verbose_name="Software Version"
+    )
+    device_types = models.ManyToManyField(to="dcim.DeviceType", related_name="+", blank=True)
+    inventory_items = models.ManyToManyField(to="dcim.InventoryItem", related_name="+", blank=True)
+    object_tags = models.ManyToManyField(to="extras.Tag", related_name="+", blank=True)
+    download_url = models.URLField(blank=True, verbose_name="Download URL")
+    image_file_checksum = models.CharField(blank=True, max_length=256, verbose_name="Image File Checksum")
+    default_image = models.BooleanField(verbose_name="Default Image", default=False)
+
+    csv_headers = [
+        "image_file_name",
+        "software",
+        "device_types",
+        "inventory_items",
+        "object_tags",
+        "download_url",
+        "image_file_checksum",
+        "default_image",
+    ]
+
+    class Meta:
+        """Meta attributes for SoftwareImageLCM."""
+
+        verbose_name = "Software Image"
+        ordering = ("software", "default_image", "image_file_name")
+        unique_together = ("image_file_name", "software")
+
+    def __str__(self):
+        """String representation of SoftwareImageLCM."""
+        msg = f"{self.image_file_name}"
+        return msg
+
+    def get_absolute_url(self):
+        """Returns the Detail view for SoftwareImageLCM models."""
+        return reverse("plugins:nautobot_device_lifecycle_mgmt:softwareimagelcm", kwargs={"pk": self.pk})
+
+    def to_csv(self):
+        """Return fields for bulk view."""
+        return (
+            self.image_file_name,
+            self.software.id,
+            f'"{",".join(str(device_type["model"]) for device_type in self.device_types.values())}"',
+            f'"{",".join(str(inventory_item["id"]) for inventory_item in self.inventory_items.values())}"',
+            f'"{",".join(str(object_tag["slug"]) for object_tag in self.object_tags.values())}"',
+            self.download_url,
+            self.image_file_checksum,
+            self.default_image,
+        )
+
+    objects = SoftwareImageLCMQuerySet.as_manager()
+
 
 class ValidatedSoftwareLCMQuerySet(RestrictedQuerySet):
     """Queryset for `ValidatedSoftwareLCM` objects."""
@@ -631,3 +730,141 @@ class ContactLCM(PrimaryModel):
             self.type,
             self.priority,
         )
+
+
+@extras_features(
+    "custom_fields",
+    "custom_links",
+    "custom_validators",
+    "export_templates",
+    "graphql",
+    "relationships",
+    "webhooks",
+    "statuses",
+)
+class CVELCM(PrimaryModel):
+    """CVELCM is a model representation of a cve vulnerability record."""
+
+    name = models.CharField(max_length=16, blank=False, unique=True)
+    published_date = models.DateField(verbose_name="Published Date")
+    link = models.URLField()
+    status = StatusField(
+        null=True,
+        blank=True,
+        on_delete=models.PROTECT,
+        to="extras.status",
+    )
+    description = models.CharField(max_length=255, blank=True, null=True)
+    severity = models.CharField(max_length=50, default=choices.CVESeverityChoices.NONE)
+    cvss = models.FloatField(blank=True, null=True, verbose_name="CVSS Base Score")
+    cvss_v2 = models.FloatField(blank=True, null=True, verbose_name="CVSSv2 Score")
+    cvss_v3 = models.FloatField(blank=True, null=True, verbose_name="CVSSv3 Score")
+    fix = models.CharField(max_length=255, blank=True, null=True)
+    comments = models.TextField(blank=True)
+
+    csv_headers = [
+        "name",
+        "published_date",
+        "link",
+        "status",
+        "description",
+        "severity",
+        "cvss",
+        "cvss_v2",
+        "cvss_v3",
+        "fix",
+        "comments",
+    ]
+
+    class Meta:
+        """Meta attributes for the class."""
+
+        verbose_name = "CVE"
+
+        ordering = ("severity", "name")
+
+    def get_absolute_url(self):
+        """Returns the Detail view for CVELCM models."""
+        return reverse("plugins:nautobot_device_lifecycle_mgmt:cvelcm", kwargs={"pk": self.pk})
+
+    def __str__(self):
+        """String representation of the model."""
+        return f"{self.name}"
+
+    def to_csv(self):
+        """Return fields for bulk view."""
+        return (
+            self.name,
+            self.published_date,
+            self.link,
+            self.status,
+            self.description,
+            self.severity,
+            self.cvss,
+            self.cvss_v2,
+            self.cvss_v3,
+            self.fix,
+            self.comments,
+        )
+
+
+@extras_features(
+    "custom_fields",
+    "custom_links",
+    "custom_validators",
+    "export_templates",
+    "graphql",
+    "relationships",
+    "webhooks",
+    "statuses",
+)
+class VulnerabilityLCM(PrimaryModel):
+    """VulnerabilityLCM is a model representation of vulnerability that affects a device."""
+
+    cve = models.ForeignKey(CVELCM, on_delete=models.CASCADE, blank=True, null=True)
+    software = models.ForeignKey(SoftwareLCM, on_delete=models.CASCADE, blank=True, null=True)
+    device = models.ForeignKey(Device, on_delete=models.CASCADE, blank=True, null=True)
+    inventory_item = models.ForeignKey(InventoryItem, on_delete=models.CASCADE, blank=True, null=True)
+    status = StatusField(
+        null=True,
+        blank=True,
+        on_delete=models.PROTECT,
+        to="extras.status",
+    )
+
+    csv_headers = [
+        "cve",
+        "software",
+        "device",
+        "inventory_item",
+        "status",
+    ]
+
+    class Meta:
+        """Meta attributes for the class."""
+
+        verbose_name = "Vulnerability"
+        verbose_name_plural = "Vulnerabilities"
+
+    def get_absolute_url(self):
+        """Returns the Detail view for VulnerabilityLCM models."""
+        return reverse("plugins:nautobot_device_lifecycle_mgmt:vulnerabilitylcm", kwargs={"pk": self.pk})
+
+    def __str__(self):
+        """String representation of the model."""
+        name = f"Device: {self.device}" if self.device else f"Inventory Part: {self.inventory_item}"
+        if self.software:
+            name += f" - Software: {self.software}"
+        if self.cve:
+            name += f" - CVE: {self.cve}"
+        return name
+
+    def to_csv(self):
+        """Return fields for bulk view."""
+        return (
+            self.cve,
+            self.software,
+            self.device,
+            self.inventory_item,
+            self.status,
+        )
