@@ -72,6 +72,31 @@ try:
                                 "nautobot_device_lifecycle_mgmt.view_softwarelcm",
                             ],
                         ),
+                        NavMenuItem(
+                            link="plugins:nautobot_device_lifecycle_mgmt:softwareimagelcm_list",
+                            name="Software Image",
+                            buttons=(
+                                NavMenuButton(
+                                    link="plugins:nautobot_device_lifecycle_mgmt:softwareimagelcm_add",
+                                    title="Add",
+                                    icon_class="mdi mdi-plus-thick",
+                                    button_class=ButtonColorChoices.GREEN,
+                                    permissions=[
+                                        "nautobot_device_lifecycle_mgmt.add_softwareimagelcm",
+                                    ],
+                                ),
+                                NavMenuButton(
+                                    link="plugins:nautobot_device_lifecycle_mgmt:softwareimagelcm_import",
+                                    title="Import",
+                                    icon_class="mdi mdi-database-import-outline",
+                                    button_class=ButtonColorChoices.BLUE,
+                                    permissions=["nautobot_device_lifecycle_mgmt.add_softwareimagelcm"],
+                                ),
+                            ),
+                            permissions=[
+                                "nautobot_device_lifecycle_mgmt.view_softwareimagelcm",
+                            ],
+                        ),
                         NavMenuItem(
                             link="plugins:nautobot_device_lifecycle_mgmt:validatedsoftwarelcm_list",
                             name="Validated Software",
@@ -97,6 +122,38 @@ try:
                                 "nautobot_device_lifecycle_mgmt.view_validatedsoftwarelcm",
                             ],
                         ),
+                        NavMenuItem(
+                            link="plugins:nautobot_device_lifecycle_mgmt:cvelcm_list",
+                            name="CVE",
+                            buttons=(
+                                NavMenuButton(
+                                    link="plugins:nautobot_device_lifecycle_mgmt:cvelcm_add",
+                                    title="Add",
+                                    icon_class="mdi mdi-plus-thick",
+                                    button_class=ButtonColorChoices.GREEN,
+                                    permissions=[
+                                        "nautobot_device_lifecycle_mgmt.add_cvelcm",
+                                    ],
+                                ),
+                                NavMenuButton(
+                                    link="plugins:nautobot_device_lifecycle_mgmt:cvelcm_import",
+                                    title="Import",
+                                    icon_class="mdi mdi-database-import-outline",
+                                    button_class=ButtonColorChoices.BLUE,
+                                    permissions=["nautobot_device_lifecycle_mgmt.add_cvelcm"],
+                                ),
+                            ),
+                            permissions=[
+                                "nautobot_device_lifecycle_mgmt.view_cvelcm",
+                            ],
+                        ),
+                        NavMenuItem(
+                            link="plugins:nautobot_device_lifecycle_mgmt:vulnerabilitylcm_list",
+                            name="Vulnerability",
+                            permissions=[
+                                "nautobot_device_lifecycle_mgmt.view_vulnerabilitylcm",
+                            ],
+                        ),
                     ),
                 ),
                 NavMenuGroup(
@@ -313,6 +370,26 @@ except ModuleNotFoundError:
                 ),
             ),
         ),
+        PluginMenuItem(
+            link="plugins:nautobot_device_lifecycle_mgmt:softwareimagelcm_list",
+            link_text="Software Image",
+            buttons=(
+                PluginMenuButton(
+                    link="plugins:nautobot_device_lifecycle_mgmt:softwareimagelcm_add",
+                    title="Add",
+                    icon_class="mdi mdi-plus-thick",
+                    color=ButtonColorChoices.GREEN,
+                    permissions=["nautobot_device_lifecycle_mgmt.add_softwareimagelcm"],
+                ),
+                PluginMenuButton(
+                    link="plugins:nautobot_device_lifecycle_mgmt:softwareimagelcm_import",
+                    title="Import",
+                    icon_class="mdi mdi-database-import-outline",
+                    color=ButtonColorChoices.BLUE,
+                    permissions=["nautobot_device_lifecycle_mgmt.add_softwareimagelcm"],
+                ),
+            ),
+        ),
         PluginMenuItem(
             link="plugins:nautobot_device_lifecycle_mgmt:validatedsoftwarelcm_list",
             link_text="Validated Software",
@@ -333,6 +410,30 @@ except ModuleNotFoundError:
                 ),
             ),
         ),
+        PluginMenuItem(
+            link="plugins:nautobot_device_lifecycle_mgmt:cvelcm_list",
+            link_text="CVE",
+            buttons=(
+                PluginMenuButton(
+                    link="plugins:nautobot_device_lifecycle_mgmt:cvelcm_add",
+                    title="Add",
+                    icon_class="mdi mdi-plus-thick",
+                    color=ButtonColorChoices.GREEN,
+                    permissions=["nautobot_device_lifecycle_mgmt.add_cvelcm"],
+                ),
+                PluginMenuButton(
+                    link="plugins:nautobot_device_lifecycle_mgmt:cvelcm_import",
+                    title="Import",
+                    icon_class="mdi mdi-database-import-outline",
+                    color=ButtonColorChoices.BLUE,
+                    permissions=["nautobot_device_lifecycle_mgmt.add_cvelcm"],
+                ),
+            ),
+        ),
+        PluginMenuItem(
+            link="plugins:nautobot_device_lifecycle_mgmt:vulnerabilitylcm_list",
+            link_text="Vulnerability",
+        ),
         PluginMenuItem(
             link="plugins:nautobot_device_lifecycle_mgmt:validatedsoftware_device_report",
             link_text="Report: Device OS Validation",
