@@ -2,9 +2,11 @@
 from datetime import date
 
 from django.test import TestCase
+from django.contrib.contenttypes.models import ContentType
 import time_machine
 
 from nautobot.dcim.models import Device, DeviceRole, DeviceType, Manufacturer, Site, Platform
+from nautobot.extras.models import Status
 
 from nautobot_device_lifecycle_mgmt.models import (
     HardwareLCM,
@@ -12,6 +14,9 @@ from nautobot_device_lifecycle_mgmt.models import (
     ValidatedSoftwareLCM,
     DeviceSoftwareValidationResult,
     InventoryItemSoftwareValidationResult,
+    CVELCM,
+    VulnerabilityLCM,
+    SoftwareImageLCM,
 )
 from nautobot_device_lifecycle_mgmt.filters import (
     HardwareLCMFilterSet,
@@ -19,8 +24,11 @@ from nautobot_device_lifecycle_mgmt.filters import (
     ValidatedSoftwareLCMFilterSet,
     DeviceSoftwareValidationResultFilterSet,
     InventoryItemSoftwareValidationResultFilterSet,
+    CVELCMFilterSet,
+    VulnerabilityLCMFilterSet,
+    SoftwareImageLCMFilterSet,
 )
-from .conftest import create_devices, create_inventory_items
+from .conftest import create_devices, create_inventory_items, create_cves, create_softwares
 
 
 class HardwareLCMTestCase(TestCase):
@@ -161,9 +169,6 @@ class SoftwareLCMFilterSetTestCase(TestCase):
                 release_date="2019-01-10",
                 end_of_support="2022-05-15",
                 documentation_url="https://www.cisco.com/c/en/us/support/ios-nx-os-software/ios-15-4m-t/series.html",
-                download_url="ftp://device-images.local.com/cisco/asr1001x-universalk9.17.03.03.SPA.bin",
-                image_file_name="asr1001x-universalk9.17.03.03.SPA.bin",
-                image_file_checksum="9cf2e09b59207a4d8ea40886fbbe5b4b68e19e58a8f96b34240e4cea9971f6ae6facab9a1855a34e1ed8755f3ffe4c969cf6e6ef1df95d42a91540a44d4b9e14",
                 long_term_support=False,
                 pre_release=True,
             ),
@@ -174,9 +179,6 @@ class SoftwareLCMFilterSetTestCase(TestCase):
                 release_date="2021-01-10",
                 end_of_support="2026-05-13",
                 documentation_url="https://www.arista.com/softdocs",
-                download_url="ftp://device-images.local.com/arista/arista-4.25m.img",
-                image_file_name="arista-4.25m.img",
-                image_file_checksum="34e61320b5518a2954b2a307b7e6a018",
                 long_term_support=True,
                 pre_release=False,
             ),
@@ -202,21 +204,6 @@ class SoftwareLCMFilterSetTestCase(TestCase):
         params = {"documentation_url": "https://www.arista.com/softdocs"}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
-    def test_download_url(self):
-        """Test download_url filter."""
-        params = {"download_url": "ftp://device-images.local.com/arista/arista-4.25m.img"}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-
-    def test_image_file_name(self):
-        """Test image_file_name filter."""
-        params = {"image_file_name": "asr1001x-universalk9.17.03.03.SPA.bin"}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-
-    def test_image_file_checksum(self):
-        """Test image_file_checksum filter."""
-        params = {"image_file_checksum": "34e61320b5518a2954b2a307b7e6a018"}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-
     def test_long_term_support(self):
         """Test long_term_support filter."""
         params = {"long_term_support": True}
@@ -477,3 +464,296 @@ class InventoryItemSoftwareValidationResultFilterSetTestCase(TestCase):
         """Test sw_missing filter."""
         params = {"exclude_sw_missing": True}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+
+class CVELCMTestCase(TestCase):
+    """Tests for CVELCMFilter."""
+
+    queryset = CVELCM.objects.all()
+    filterset = CVELCMFilterSet
+
+    def setUp(self):
+        cve_ct = ContentType.objects.get_for_model(CVELCM)
+        fixed = Status.objects.create(name="Fixed", slug="fixed", color="4caf50", description="Unit has been fixed")
+        fixed.content_types.set([cve_ct])
+        not_fixed = Status.objects.create(
+            name="Not Fixed", slug="not-fixed", color="f44336", description="Unit is not fixed"
+        )
+        not_fixed.content_types.set([cve_ct])
+
+        CVELCM.objects.create(
+            name="CVE-2021-1391",
+            published_date="2021-03-24",
+            link="https://www.cvedetails.com/cve/CVE-2021-1391/",
+            cvss=3,
+            cvss_v2=3,
+            cvss_v3=3,
+        )
+        CVELCM.objects.create(
+            name="CVE-2021-44228",
+            published_date="2021-12-10",
+            link="https://www.cvedetails.com/cve/CVE-2021-44228/",
+            status=not_fixed,
+            cvss=5,
+            cvss_v2=5,
+            cvss_v3=5,
+        )
+        CVELCM.objects.create(
+            name="CVE-2020-27134",
+            published_date="2020-12-11",
+            link="https://www.cvedetails.com/cve/CVE-2020-27134/",
+            severity="Critical",
+            status=fixed,
+            cvss=7,
+            cvss_v2=7,
+            cvss_v3=7,
+        )
+
+    def test_q_one_year(self):
+        """Test q filter to find single record based on year."""
+        params = {"q": "2020"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+    def test_q_two_year(self):
+        """Test q filter to find two records based on year."""
+        params = {"q": "2021"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_q_link(self):
+        """Test q filter to all records from link."""
+        params = {"q": "cvedetails"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+
+    def test_status(self):
+        """Test status filter."""
+        params = {"status": ["fixed"]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+    def test_exclude_status(self):
+        """Test exclude_status filter."""
+        params = {"exclude_status": ["fixed"]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_severity(self):
+        """Test severity filter."""
+        params = {"severity": "Critical"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+    def test_published_date_before(self):
+        """Test published_date_before filter."""
+        params = {"published_date_before": "2021-01-01"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+    def test_published_date_after(self):
+        """Test published_date_after filter."""
+        params = {"published_date_after": "2021-01-01"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_cvss_gte(self):
+        """Test cvss__gte filter."""
+        params = {"cvss__gte": 1}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        params = {"cvss__gte": 4}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        params = {"cvss__gte": 6}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        params = {"cvss__gte": 9}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 0)
+
+    def test_cvss_lte(self):
+        """Test cvss__lte filter."""
+        params = {"cvss__lte": 1}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 0)
+        params = {"cvss__lte": 4}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        params = {"cvss__lte": 6}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        params = {"cvss__lte": 9}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+
+    def test_cvss_v2_gte(self):
+        """Test cvss_v2__gte filter."""
+        params = {"cvss_v2__gte": 1}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        params = {"cvss_v2__gte": 4}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        params = {"cvss_v2__gte": 6}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        params = {"cvss_v2__gte": 9}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 0)
+
+    def test_cvss_v2_lte(self):
+        """Test cvss_v2__lte filter."""
+        params = {"cvss_v2__lte": 1}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 0)
+        params = {"cvss_v2__lte": 4}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        params = {"cvss_v2__lte": 6}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        params = {"cvss_v2__lte": 9}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+
+    def test_cvss_v3_gte(self):
+        """Test cvss_v3__gte filter."""
+        params = {"cvss_v3__gte": 1}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        params = {"cvss_v3__gte": 4}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        params = {"cvss_v3__gte": 6}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        params = {"cvss_v3__gte": 9}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 0)
+
+    def test_cvss_v3_lte(self):
+        """Test cvss_v3__lte filter."""
+        params = {"cvss_v3__lte": 1}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 0)
+        params = {"cvss_v3__lte": 4}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        params = {"cvss_v3__lte": 6}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        params = {"cvss_v3__lte": 9}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+
+
+class VulnerabilityLCMTestCase(TestCase):
+    """Tests for VulnerabilityLCMFilter."""
+
+    queryset = VulnerabilityLCM.objects.all()
+    filterset = VulnerabilityLCMFilterSet
+
+    def setUp(self):
+        inventory_items = create_inventory_items()
+        cves = create_cves()
+        softwares = create_softwares()
+        vuln_ct = ContentType.objects.get_for_model(VulnerabilityLCM)
+        fix_me = Status.objects.create(name="Fix Me", slug="fix_me", color="4caf50", description="Please fix me.")
+        fix_me.content_types.set([vuln_ct])
+
+        VulnerabilityLCM.objects.create(
+            cve=cves[0],
+            device=inventory_items[0].device,
+            software=softwares[0],
+        )
+        VulnerabilityLCM.objects.create(
+            cve=cves[1],
+            device=inventory_items[1].device,
+            software=softwares[1],
+            status=fix_me,
+        )
+        VulnerabilityLCM.objects.create(
+            cve=cves[2],
+            inventory_item=inventory_items[2],
+            software=softwares[2],
+            status=fix_me,
+        )
+
+    def test_q_cve(self):
+        """Test q filter to find single record based on CVE name."""
+        params = {"q": "2020"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+    def test_q_device(self):
+        """Test q filter to find single record based on Device name."""
+        params = {"q": "sw1"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+    def test_q_inventory_item(self):
+        """Test q filter to find single record based on Inventory Item name."""
+        params = {"q": "48x RJ-45"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+    def test_q_software_device_platform(self):
+        """Test q filter to find single record based on Software Device Platform name."""
+        params = {"q": "Cisco IOS"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+
+    def test_q_software_version(self):
+        """Test q filter to find single record based on Software version."""
+        params = {"q": "4.22.9M"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+
+class SoftwareImageLCMFilterSetTestCase(TestCase):
+    """Tests for SoftwareImageLCMFilterSet."""
+
+    queryset = SoftwareImageLCM.objects.all()
+    filterset = SoftwareImageLCMFilterSet
+
+    def setUp(self):
+        manufacturer_cisco, _ = Manufacturer.objects.get_or_create(name="Cisco", slug="cisco")
+        manufacturer_arista, _ = Manufacturer.objects.get_or_create(name="Arista", slug="arista")
+        device_platform_cisco, _ = Platform.objects.get_or_create(
+            name="Cisco IOS", slug="cisco_ios", manufacturer=manufacturer_cisco
+        )
+        device_platform_arista, _ = Platform.objects.get_or_create(
+            name="Arista EOS", slug="arista_eos", manufacturer=manufacturer_arista
+        )
+
+        self.softwares = (
+            SoftwareLCM.objects.create(
+                device_platform=device_platform_cisco,
+                version="17.3.3 MD",
+                release_date="2019-01-10",
+            ),
+            SoftwareLCM.objects.create(
+                device_platform=device_platform_arista,
+                version="4.25M",
+                release_date="2021-01-10",
+            ),
+        )
+
+        devicetype_1, _ = DeviceType.objects.get_or_create(
+            manufacturer=manufacturer_cisco, model="ASR-1000", slug="asr-1000"
+        )
+        self.devicetype_2, _ = DeviceType.objects.get_or_create(
+            manufacturer=manufacturer_arista, model="7150S", slug="7150s"
+        )
+
+        soft_image = SoftwareImageLCM(
+            image_file_name="ios17.3.3md.img",
+            software=self.softwares[0],
+            default_image=True,
+        )
+        soft_image.save()
+
+        soft_image = SoftwareImageLCM(
+            image_file_name="ios17.3.3md-ssl.img",
+            software=self.softwares[0],
+            default_image=False,
+        )
+        soft_image.device_types.set([devicetype_1.pk])
+        soft_image.save()
+
+        soft_image = SoftwareImageLCM(
+            image_file_name="eos4.25.m.swi",
+            software=self.softwares[1],
+            default_image=True,
+        )
+        soft_image.device_types.set([self.devicetype_2.pk])
+        soft_image.save()
+
+    def test_q_image_name(self):
+        """Test q filter to find single record based on the image name."""
+        params = {"q": "ios17.3.3"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_q_soft_version(self):
+        """Test q filter to find single record based on the software version."""
+        params = {"q": "4.25M"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+    def test_software(self):
+        """Test software filter."""
+        params = {"software": [self.softwares[0].pk]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_default_image(self):
+        """Test default_image filter."""
+        params = {"default_image": True}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_device_types(self):
+        """Test device_types filter."""
+        params = {"device_types": [self.devicetype_2.model]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
