@@ -8,6 +8,8 @@ from django.contrib.contenttypes.models import ContentType
 import time_machine
 
 from nautobot.dcim.models import DeviceType, Manufacturer, Platform
+from nautobot.extras.choices import RelationshipTypeChoices
+from nautobot.extras.models import Relationship, RelationshipAssociation, Status, Tag
 
 from nautobot_device_lifecycle_mgmt.models import (
     HardwareLCM,
@@ -15,11 +17,14 @@ from nautobot_device_lifecycle_mgmt.models import (
     SoftwareLCM,
     ValidatedSoftwareLCM,
     DeviceSoftwareValidationResult,
+    CVELCM,
+    VulnerabilityLCM,
+    SoftwareImageLCM,
 )
-from .conftest import create_devices, create_inventory_items
+from .conftest import create_devices, create_inventory_items, create_cves, create_softwares
 
 
-class TestModelBasic(TestCase):
+class HardwareLCMTestCase(TestCase):
     """Tests for the HardwareLCM models."""
 
     def setUp(self):
@@ -149,9 +154,6 @@ class SoftwareLCMTestCase(TestCase):
             release_date=date(2019, 1, 10),
             end_of_support=date(2022, 5, 15),
             documentation_url="https://www.cisco.com/c/en/us/support/ios-nx-os-software/ios-15-4m-t/series.html",
-            download_url="ftp://device-images.local.com/cisco/asr1001x-universalk9.17.03.03.SPA.bin",
-            image_file_name="asr1001x-universalk9.17.03.03.SPA.bin",
-            image_file_checksum="9cf2e09b59207a4d8ea40886fbbe5b4b68e19e58a8f96b34240e4cea9971f6ae6facab9a1855a34e1ed8755f3ffe4c969cf6e6ef1df95d42a91540a44d4b9e14",
             long_term_support=False,
             pre_release=True,
         )
@@ -165,14 +167,6 @@ class SoftwareLCMTestCase(TestCase):
             softwarelcm_full.documentation_url,
             "https://www.cisco.com/c/en/us/support/ios-nx-os-software/ios-15-4m-t/series.html",
         )
-        self.assertEqual(
-            softwarelcm_full.download_url, "ftp://device-images.local.com/cisco/asr1001x-universalk9.17.03.03.SPA.bin"
-        )
-        self.assertEqual(softwarelcm_full.image_file_name, "asr1001x-universalk9.17.03.03.SPA.bin")
-        self.assertEqual(
-            softwarelcm_full.image_file_checksum,
-            "9cf2e09b59207a4d8ea40886fbbe5b4b68e19e58a8f96b34240e4cea9971f6ae6facab9a1855a34e1ed8755f3ffe4c969cf6e6ef1df95d42a91540a44d4b9e14",
-        )
         self.assertEqual(softwarelcm_full.long_term_support, False)
         self.assertEqual(softwarelcm_full.pre_release, True)
         self.assertEqual(str(softwarelcm_full), f"{self.device_platform.name} - {softwarelcm_full.version}")
@@ -312,3 +306,201 @@ class InventoryItemSoftwareValidationResultTestCase(TestCase):
         self.assertEqual(validation_result.inventory_item, self.inventory_item)
         self.assertEqual(validation_result.software, self.software)
         self.assertEqual(validation_result.is_validated, True)
+
+
+class CVELCMTestCase(TestCase):
+    """Tests for the CVELCM model."""
+
+    def setUp(self):
+        """Set up the test objects."""
+        self.device_platform = Platform.objects.create(name="Cisco IOS", slug="cisco_ios")
+        self.softwarelcm = SoftwareLCM.objects.create(device_platform=self.device_platform, version="15.2(5)e")
+        self.cve_ct = ContentType.objects.get_for_model(CVELCM)
+        self.software_ct = ContentType.objects.get_for_model(SoftwareLCM)
+        self.relationship = Relationship.objects.get_or_create(
+            name="CVE to Software",
+            defaults={
+                "name": "CVE to Software",
+                "slug": "cve_soft",
+                "type": RelationshipTypeChoices.TYPE_MANY_TO_MANY,
+                "source_type": ContentType.objects.get_for_model(CVELCM),
+                "source_label": "Affected Softwares",
+                "destination_type": ContentType.objects.get_for_model(SoftwareLCM),
+                "destination_label": "Corresponding CVEs",
+            },
+        )[0]
+        self.status = Status.objects.create(
+            name="Fixed", slug="fixed", color="4caf50", description="Unit has been fixed"
+        )
+        self.status.content_types.set([self.cve_ct])
+
+    def test_create_cvelcm_required_only(self):
+        """Successfully create CVELCM with required fields only."""
+        cvelcm = CVELCM.objects.create(
+            name="CVE-2021-1391", published_date="2021-03-24", link="https://www.cvedetails.com/cve/CVE-2021-1391/"
+        )
+
+        self.assertEqual(cvelcm.name, "CVE-2021-1391")
+        self.assertEqual(cvelcm.published_date, "2021-03-24")
+        self.assertEqual(cvelcm.link, "https://www.cvedetails.com/cve/CVE-2021-1391/")
+
+    def test_create_cvelcm_all(self):
+        """Successfully create CVELCM with all fields."""
+        cvelcm = CVELCM.objects.create(
+            name="CVE-2021-34699",
+            published_date="2021-09-23",
+            link="https://www.cvedetails.com/cve/CVE-2021-34699/",
+            status=self.status,
+            description="Thanos",
+            severity="High",
+            cvss=6.8,
+            cvss_v2=6.9,
+            cvss_v3=6.7,
+            fix="Avengers",
+            comments="This is very bad juju.",
+        )
+
+        self.assertEqual(cvelcm.name, "CVE-2021-34699")
+        self.assertEqual(cvelcm.published_date, "2021-09-23")
+        self.assertEqual(cvelcm.link, "https://www.cvedetails.com/cve/CVE-2021-34699/")
+        self.assertEqual(cvelcm.status, self.status)
+        self.assertEqual(cvelcm.description, "Thanos")
+        self.assertEqual(cvelcm.severity, "High")
+        self.assertEqual(cvelcm.cvss, 6.8)
+        self.assertEqual(cvelcm.cvss_v2, 6.9)
+        self.assertEqual(cvelcm.cvss_v3, 6.7)
+        self.assertEqual(cvelcm.fix, "Avengers")
+        self.assertEqual(cvelcm.comments, "This is very bad juju.")
+
+    def test_create_cve_soft_relationship_association(self):
+        """Successfully create a relationship between CVE and Software."""
+        cvelcm = CVELCM.objects.create(
+            name="CVE-2021-1391", published_date="2021-03-24", link="https://www.cvedetails.com/cve/CVE-2021-1391/"
+        )
+
+        association = RelationshipAssociation.objects.create(
+            relationship_id=self.relationship.id,
+            source_id=cvelcm.id,
+            source_type_id=self.cve_ct.id,
+            destination_id=self.softwarelcm.id,
+            destination_type_id=self.software_ct.id,
+        )
+
+        cve_rels = cvelcm.get_relationships()
+
+        self.assertEqual(cve_rels["source"][self.relationship].first(), association)
+
+
+class VulnerabilityLCMTestCase(TestCase):
+    """Tests for the VulnerabilityLCM model."""
+
+    def setUp(self):
+        """Set up the test objects."""
+        self.inv_items = create_inventory_items()
+        self.devices = [inv_item.device for inv_item in self.inv_items]
+        self.cves = create_cves()
+        self.softwares = create_softwares()
+        vuln_ct = ContentType.objects.get_for_model(VulnerabilityLCM)
+        self.status = Status.objects.create(
+            name="Exempt", slug="exempt", color="4caf50", description="This unit is exempt."
+        )
+        self.status.content_types.set([vuln_ct])
+
+    def test_create_vulnerabilitylcm_device_required_only(self):
+        """Successfully create VulnerabilityLCM with required fields only."""
+        vulnerability = VulnerabilityLCM.objects.create(
+            cve=self.cves[0], software=self.softwares[0], device=self.devices[0]
+        )
+
+        self.assertEqual(str(vulnerability), "Device: sw1 - Software: Cisco IOS - 15.1(2)M - CVE: CVE-2021-1391")
+        self.assertEqual(vulnerability.cve, self.cves[0])
+        self.assertEqual(vulnerability.software, self.softwares[0])
+        self.assertEqual(vulnerability.device, self.devices[0])
+
+    def test_create_vulnerabilitylcm_inventory_item_required_only(self):
+        """Successfully create VulnerabilityLCM with required fields only."""
+        vulnerability = VulnerabilityLCM.objects.create(
+            cve=self.cves[1], software=self.softwares[1], inventory_item=self.inv_items[1]
+        )
+
+        self.assertEqual(
+            str(vulnerability),
+            "Inventory Part: 100GBASE-SR4 QSFP Transceiver - Software: Cisco IOS - 4.22.9M - CVE: CVE-2021-44228",
+        )
+        self.assertEqual(vulnerability.cve, self.cves[1])
+        self.assertEqual(vulnerability.software, self.softwares[1])
+        self.assertEqual(vulnerability.inventory_item, self.inv_items[1])
+
+    def test_create_vulnerabilitylcm_all(self):
+        """Successfully create VulnerabilityLCM with all fields."""
+        vulnerability = VulnerabilityLCM.objects.create(
+            cve=self.cves[2], software=self.softwares[2], device=self.devices[2], status=self.status
+        )
+
+        self.assertEqual(str(vulnerability), "Device: sw3 - Software: Cisco IOS - 21.4R3 - CVE: CVE-2020-27134")
+        self.assertEqual(vulnerability.cve, self.cves[2])
+        self.assertEqual(vulnerability.software, self.softwares[2])
+        self.assertEqual(vulnerability.device, self.devices[2])
+        self.assertEqual(vulnerability.status, self.status)
+
+
+class SoftwareImageLCMTestCase(TestCase):
+    """Tests for the SoftwareImageLCM model."""
+
+    def setUp(self):
+        """Set up base objects."""
+        device_platform = Platform.objects.get_or_create(name="Cisco IOS", slug="cisco_ios")[0]
+        self.software = SoftwareLCM.objects.create(
+            device_platform=device_platform,
+            version="17.3.3 MD",
+            release_date=date(2019, 1, 10),
+        )
+        manufacturer = Manufacturer.objects.create(name="Cisco", slug="cisco")
+        self.device_type_1 = DeviceType.objects.create(manufacturer=manufacturer, model="ASR-1000", slug="asr-1000")
+        self.device_type_2 = DeviceType.objects.create(manufacturer=manufacturer, model="CAT-3750", slug="cat-3750")
+        self.inventory_item = create_inventory_items()[0]
+        self.tag = Tag.objects.create(name="asr", slug="asr")
+
+    def test_create_softwareimage_required_only(self):
+        """Successfully create SoftwareImageLCM with required fields only."""
+        softwareimage = SoftwareImageLCM(image_file_name="ios17.3.3md.img", software=self.software)
+        softwareimage.device_types.set([self.device_type_1])
+        softwareimage.save()
+
+        self.assertEqual(softwareimage.image_file_name, "ios17.3.3md.img")
+        self.assertEqual(softwareimage.software, self.software)
+        self.assertEqual(list(softwareimage.device_types.all()), [self.device_type_1])
+
+    def test_create_softwareimage_all(self):
+        """Successfully create SoftwareImageLCM with all fields."""
+        softwareimage = SoftwareImageLCM(
+            image_file_name="ios17.3.3md.img",
+            software=self.software,
+            download_url="ftp://images.local/cisco/ios17.3.3md.img",
+            image_file_checksum="441rfabd75b0512r7fde7a7a66faa596",
+            default_image=True,
+        )
+        softwareimage.device_types.set([self.device_type_1])
+        softwareimage.inventory_items.set([self.inventory_item])
+        softwareimage.object_tags.set([self.tag])
+        softwareimage.save()
+
+        self.assertEqual(softwareimage.image_file_name, "ios17.3.3md.img")
+        self.assertEqual(softwareimage.software, self.software)
+        self.assertEqual(softwareimage.download_url, "ftp://images.local/cisco/ios17.3.3md.img")
+        self.assertEqual(softwareimage.image_file_checksum, "441rfabd75b0512r7fde7a7a66faa596")
+        self.assertEqual(softwareimage.default_image, True)
+        self.assertEqual(list(softwareimage.device_types.all()), [self.device_type_1])
+        self.assertEqual(list(softwareimage.inventory_items.all()), [self.inventory_item])
+        self.assertEqual(list(softwareimage.object_tags.all()), [self.tag])
+        self.assertEqual(str(softwareimage), f"{softwareimage.image_file_name}")
+
+    def test_validatedsoftwarelcm_valid_property(self):
+        """Test behavior of the 'valid' property."""
+        validatedsoftwarelcm_start_only = ValidatedSoftwareLCM(
+            software=self.software,
+            start=date(2020, 4, 15),
+            preferred=False,
+        )
+        validatedsoftwarelcm_start_only.device_types.set([self.device_type_1])
+        validatedsoftwarelcm_start_only.save()
